// Handle GET JSON request
function getJSON() {
    // Show response panel
    $("#response-json").html("Loading...");
    $('#response-json-panel').show();
    // Scroll to the bottom of the page
    window.scrollTo({ top: document.body.scrollHeight, behavior: 'smooth' });

    // Get active query element
    var active_query_input = $("#queries-input").find(".tab-pane.active").find(".query-input.active");
    var active_query_function = active_query_input.data("query-func");

    // Prepare query
    var query_data = {type: "json"};
    if (active_query_function == "/custom") {
        // Get query from editor
        var editor = ace.edit("query-editor");
        query_data["query"] = editor.getValue();

        // Store query in the local storage
        localStorage.last_query = editor.getValue();
    } else {
        active_query_input.find("input").each(function(){
            if ($(this).val()) {
                query_data[$(this).attr("name")] = $(this).val();
            } else {
                $(this).addClass("input-error");
                $(this).val("Missing input");
                $('#response-json-panel').hide();
                return false;
            }
        });
    };

    // Make a query and show result
    if (active_query_function == "/custom") {
        $.post(localStorage.api_url + active_query_function, JSON.stringify(query_data), function(result){
            $("#response-json").jsonViewer(result.response, {rootCollapsable: false, withLinks: false});
        }).fail(function(result) {
            var error_text = "Error " + result.status + " (" + result.status_text + "): " + result.responseText;
            $("#response-json").html(error_text);
        });
    } else {
        $.get(localStorage.api_url + active_query_function, query_data, function(result){
            $("#response-json").jsonViewer(result.response, {rootCollapsable: false, withLinks: false});
        }).fail(function(result) {
            var error_text = "Error " + result.status + " (" + result.status_text + "): " + result.responseText;
            $("#response-json").html(error_text);
        });
    }
};


var network = NaN
function drawGraph(response) {
    var nodes = new vis.DataSet(response.nodes);
    var edges = new vis.DataSet(response.edges);
    var container = document.getElementById('network-graph');
    var data = {
        nodes: nodes,
        edges: edges
    };
    var options = {
        nodes: {
            shape: 'dot',
            size: 30,
            borderWidth: 1,
            font: {
                size: 34
            }
        },
        edges: {
            smooth: {
                type: "discrete",
            },
            width: 3,
            font: {
                size: 34
            },
            color: {
                inherit: 'to'
            }
        },
        layout: {
            improvedLayout: true
        },
        physics: {
            enabled: true,
            stabilization: {
                enabled: true,
                iterations: 100
            },
            repulsion: {
                centralGravity: 0,
                springLength: 500,
                damping: 0.1,
            },
            solver: "repulsion"
        },
        interaction: {
            tooltipDelay: 0
        },
    };
    network = new vis.Network(container, data, options);

    // Fix node position after it is moved (otherwise the physics will move it back)
    network.on('dragEnd', function(properties) {
        var ids = properties.nodes;
        if (network.isCluster(ids[0]) == true) {
            network.updateClusteredNode(ids[0], {fixed: true});
        } else {
            var clicked_node = nodes.get(ids)[0];
            if (clicked_node) {
                nodes.update({id: clicked_node.id, fixed: true});
            }; 
        };
    });
    network.on('dragStart', function(properties) {
        var ids = properties.nodes;
        if (network.isCluster(ids[0]) == true) {
            network.updateClusteredNode(ids[0], {fixed: false});
        } else {
            var clicked_node = nodes.get(ids)[0];
            if (clicked_node) {
                nodes.update({id: clicked_node.id, fixed: false});
            };
        };
    });

    // Show and hide node info card when node is selected and deselected
    network.on('selectNode', function(properties) {
        var ids = properties.nodes;
        if (network.isCluster(ids[0]) != true) {
            var clicked_node = nodes.get(ids)[0];
            $('#network-graph-info-data').html(clicked_node.title);
            $('#network-graph-info-node-id').html(clicked_node.id);
            $('#network-graph-info-node-label').html(clicked_node.label);
            $('#network-graph-info').show();
        }
    });
    network.on('deselectNode', function(properties) {
        $('#network-graph-info-data').html('');
        $('#network-graph-info').hide();
    });
}

 // Node Clustering function
function clusterNode(onlyOutliers){
    node_id = $('#network-graph-info-node-id').html();
    var options = {
        joinCondition: function(parentNodeOptions, childNodeOptions) {
            return (onlyOutliers) ? childNodeOptions.amountOfConnections === 1 : true;
        },
        clusterNodeProperties: {
            label: $('#network-graph-info-node-label').html(),
            shape: "square",
            fixed: true
        }
    }
    network.clusterByConnection(node_id, options);

    network.on("doubleClick", function (properties) {
        if (properties.nodes.length === 1) {
            var ids = properties.nodes;
            if (network.isCluster(ids[0]) == true) {
                network.openCluster(ids[0]);
            };
        };
    });
};


// Handle GET Graph request
function getGraph() {
    // Reset info card
    $('#network-graph-info-data').html('');
    $('#network-graph-info').hide();

    // Show response panel with loading information
    $('#network-graph').html(' \
        <div class="loading-info"> \
            <i class="fas fa-circle-notch fa-spin"></i><span class="loading-text">Loading ...</span> \
        </div>'
    )
    $('#response-graph-panel').show();
    // Scroll to the bottom of the page
    window.scrollTo({ top: document.body.scrollHeight, behavior: 'smooth' });

    // Get active query element
    var active_query_input = $("#queries-input").find(".tab-pane.active").find(".query-input.active");
    var active_query_function = active_query_input.data("query-func");

    // Prepare query
    var query_data = {type: "graph"};
    if (active_query_function == "/custom") {
        // Get query from editor
        var editor = ace.edit("query-editor");
        query_data["query"] = editor.getValue();

        // Store query in the local storage
        localStorage.last_query = editor.getValue();
    } else {
        active_query_input.find("input").each(function(){
            if ($(this).val()) {
                query_data[$(this).attr("name")] = $(this).val();
            } else {
                $(this).addClass("input-error");
                $(this).val("Missing input");
                $('#response-graph-panel').hide();
                return false;
            }
        });
    };

    // Make a query and show result
    if (active_query_function == "/custom") {
        $.post(localStorage.api_url + active_query_function, JSON.stringify(query_data), function(result){
            drawGraph(result.response);
        }).fail(function(result) {
            var error_text = "Error " + result.status + " (" + result.status_text + "): " + result.responseText;
            var error_info = ' \
                <div class="loading-info"> \
                    <i class="fas fa-exclamation-triangle"></i><span class="loading-text">' + error_text + '</span> \
                </div>'
            $('#network-graph').html(error_info)
        });
    } else {
        $.get(localStorage.api_url + active_query_function, query_data, function(result){
            drawGraph(result.response);
        }).fail(function(result) {
            var error_text = "Error " + result.status + " (" + result.status_text + "): " + result.responseText;
            var error_info = ' \
                <div class="loading-info"> \
                    <i class="fas fa-exclamation-triangle"></i><span class="loading-text">' + error_text + '</span> \
                </div>'
            $('#network-graph').html(error_info);
        });
    }
}


function reloadGraphInfoData() {
    // Relode info only if there is info displayed
    if ($('#network-graph-info-data').html()) {
        var selected_node = network.body.nodes[network.getSelectedNodes()];
        $('#network-graph-info-data').html(selected_node.options.title);
    };
};


function appendToGraph(query_function, query_data) {
    // Make a query and append result
    $.get(localStorage.api_url + query_function, query_data, function(result){
        result.response.nodes.forEach(item => {
            if (network.body.nodeIndices.includes(item.id)) {
                // Remove location data from an existing item
                delete item.x;
                delete item.y;

                network.body.data.nodes.update(result.response.nodes);
            } else {
                network.body.data.nodes.add(item);
            };
        });
        result.response.edges.forEach(item => {
            if (network.body.edgeIndices.includes(item.id)) {
                network.body.data.edges.update(result.response.edges);
            } else {
                network.body.data.edges.add(item);
            };
        });
        // Reload displayed info
        reloadGraphInfoData();
    }).fail(function(result) {
        var error_text = "Error " + result.status + " (" + result.status_text + "): " + result.responseText;
        var error_info = ' \
            <div class="loading-info"> \
                <i class="fas fa-exclamation-triangle"></i><span class="loading-text">' + error_text + '</span> \
            </div>';
        $('#network-graph').html(error_info);
    });
}


// Register button calls
$(window).on('load', function() {
    $('#btn-get-json').click(function(){
        getJSON();
        return false;
    });
    $('#btn-get-graph').click(function(){
        getGraph();
        return false;
    });
});
