// Ace editor initialization
function initAce() {
    var editor = ace.edit("query-editor");
    editor.setTheme("ace/theme/tomorrow");
    editor.session.setMode("ace/mode/d");

    // Insert the last query if it is stored in the local storage
    if (localStorage.last_query) {
        editor.insert(localStorage.last_query);
    };
};


// Handle modal status erase
function handleStatusErase() {
    $('#modal-settings').on('hidden.bs.modal', function () {
        $("#settings-reconnect-status").empty();
    })
};


function closePanel() {
    $(".close-panel").click(function() {
        // Clear response data
        var response_element = $(this).closest(".response-panel").find(".response");
        if (response_element.is("textarea")) {
            response_element.val("");
        } else {
            response_element.empty();
        }
        // Hide panel
        $(this).closest(".response-panel").hide();
    });
};


// Content loading for generic form modals
function loadFormModal() {
    $('.modal-form').on('show.bs.modal', function(e) {
        var modal = $(this);
        var modal_id = $(e.relatedTarget).data("id");
        modal.find('.modal-header').load('./modals/' + modal_id + '.html #' + modal_id + '-header');
        modal.find('.modal-body').load('./modals/' + modal_id + '.html #' + modal_id + '-body');
        $.get('./modals/' + modal_id + '.html', function(html){
            var doc = $(html);
            modal.find('.modal-scripts').empty().append(doc.filter('#' + modal_id + '-scripts'));
        });
    });
};


// Clear input with error on user click
function clearErrorInput() {
    // JQuery magic to handle events on dynamic elements
    $('form').on('click', '.input-error', function(){
        $(this).removeClass('input-error');
        $(this).val("");
    });
};


// Load page parameter granef_api or show settings modal
function loadConfiguration() {
    // Load page parameter granef_api
    var searchParams = new URLSearchParams(window.location.search);
    if (searchParams.has('granef_api')) {
        localStorage.api_url = searchParams.get('granef_api').replace(/\/$/, '');
        $("#api-url").val(localStorage.api_url);
    };
    // Show configuration modal on the firt page visit if api_url is not defined
    if (localStorage.getItem("api_url") === null) {
        $('#settings').click();
    };
};


// Init all script on window load
$(window).on("load", function() {
    initAce();
    handleStatusErase();
    closePanel();
    loadFormModal();
    clearErrorInput();
    loadConfiguration();
});
