package main

import (
        "net"
        "strings"
)

func Tokenizer() interface{} { return CIDRTokenizer{} }

type CIDRTokenizer struct{}

func (CIDRTokenizer) Name() string     { return "cidr" }
func (CIDRTokenizer) Type() string     { return "string" }
func (CIDRTokenizer) Identifier() byte { return 0xff }

func (t CIDRTokenizer) Tokens(value interface{}) ([]string, error) {
        input := value.(string)

	// _0 -> : because IPv6 adresses are saved with _0 instead of : in 
	// the database (Data Transformation)
	if strings.Contains(input, "_0") {
                input = strings.ReplaceAll(input, "_0", ":")
        }

        if  !strings.Contains(input, "/") {
                if strings.Contains(input, ":") {
                        input = input + "/128"
                } else {
                        input = input + "/32"
                }
        }

        _, ipnet, err := net.ParseCIDR(input)

        if err != nil {
                return nil, err
        }

        ones, bits := ipnet.Mask.Size()
        var toks []string

        for i := ones; i >= 1; i-- {
                m := net.CIDRMask(i, bits)
                tok := net.IPNet{
                        IP:   ipnet.IP.Mask(m),
                        Mask: m,
                }

                toks = append(toks, tok.String())
        }

        return toks, nil
}
